from enum import Enum

from ocsf.objects.resource_details import ResourceDetails
from ocsf.objects.actor import Actor
from ocsf.objects.database import Database
from ocsf.objects.databucket import Databucket
from ocsf.objects.data_security import DataSecurity
from ocsf.objects.device import Device
from ocsf.objects.file import File
from ocsf.objects.network_endpoint import NetworkEndpoint
from ocsf.objects.table import Table

from ocsf.events.findings import Finding


class DataSecurityFindingActivityId(Enum):
    """
    The normalized identifier of the Data Security Finding activity.
    """
    Create: int = 1
    Update: int = 2
    Close: int = 3
    Suppressed: int = 4

class DataSecurityFinding(Finding):
    """
    A Data Security Finding describes detections or alerts generated by various data
    security products such as Data Loss Prevention (DLP), Data Classification,
    Secrets Management, Digital Rights Management (DRM), Data Security Posture
    Management (DSPM), and similar tools. These detections or alerts can be created
    using fingerprinting, statistical analysis, machine learning or other
    methodologies. The finding describes the actors and endpoints who accessed or
    own the sensitive data, as well as the resources which store the sensitive data.
    """

    class_uid: int = 2006
    class_name: str = 'Data Security Finding'

    activity_id: DataSecurityFindingActivityId # The normalized identifier of the Data
                                               # Security Finding activity.

    # Recommended:
    resources: list[ResourceDetails] | None = None # Describes details about resources twhere
                                                   # classified or sensitive data is stored in, or was
                                                   # accessed from.
    actor: Actor | None = None # Describes details about the actor implicated in the data security
                               # finding. Either an actor that owns a particular digital file or
                               # information store, or an actor which accessed classified or sensitive
                               # data.
    database: Database | None = None # Describes the database where classified or sensitive data is
                                     # stored in, or was accessed from. Databases are typically
                                     # datastore services that contain an organized collection of
                                     # structured and/or semi-structured data.
    databucket: Databucket | None = None # Describes the databucket where classified or sensitive data
                                         # is stored in, or was accessed from. The data bucket object
                                         # is a basic container that holds data, typically organized
                                         # through the use of data partitions.
    table: Table | None = None # Describes the table where classified or sensitive data is stored in,
                               # or was accessed from. The table object represents a table within a
                               # structured relational database, warehouse, lake, or similar.
    device: Device | None = None # Describes the device where classified or sensitive data is stored
                                 # in, or was accessed from.
    dst_endpoint: NetworkEndpoint | None = None # Describes the endpoint where classified or sensitive
                                                # data is stored in, or was accessed from.
    file: File | None = None # Describes a file that contains classified or sensitive data.
    src_endpoint: NetworkEndpoint | None = None # Details about the source endpoint where classified or
                                                # sensitive data was accessed from.
    data_security: DataSecurity | None = None

    # Optional:
    activity_name: str | None = None # The Data Security finding activity name, as defined by the
                                     # `activity_id`.
    impact: str | None = None
    impact_id: int | None = None
    impact_score: int | None = None
    risk_level: str | None = None
    risk_level_id: int | None = None
    risk_score: int | None = None
